import {
  nestMap,
  fetchSync,
  DomNode,
  MultipleDomSelector,
  NestArray,
  SingleDomSelector,
} from "gigantes";

type DomSelector = SingleDomSelector | MultipleDomSelector<NestArray<DomNode>>;

export const loadDomSelector = (path: string) => {
  const module = require(path);
  const domSelectors: { name: string; domSelector: DomSelector }[] = [];
  Object.keys(module).forEach((key) => {
    if (
      ["SingleDomSelector", "MultipleDomSelector"].includes(
        module[key].constructor.name
      )
    ) {
      delete require.cache[path];
      domSelectors.push({
        name: key,
        domSelector: grantType(module[key]),
      });
    }
  });

  return domSelectors;
};

const grantType = (domSelector: any) => {
  const source: any = {
    type: domSelector.constructor.name,
  };

  if (domSelector.prevElementSpec) {
    source.prevElementSpec = grantType(domSelector.prevElementSpec);
  }

  if (domSelector.parentElementSpec) {
    source.parentElementSpec = grantType(domSelector.parentElementSpec);
  }

  return Object.assign(domSelector, source);
};

const getDomNode = (
  domSelectors: DomSelector[],
  body: DomNode
): NestArray<DomNode> => {
  let domNodes: NestArray<DomNode> = [body];

  domSelectors.forEach((domSelector) => {
    domNodes = nestMap(domNodes, (domNode: DomNode) => {
      const domNodes = domSelector.getDomNode()(domNode) as NestArray<DomNode>;
      if (Array.isArray(domNodes)) {
        return domNodes;
      } else {
        return [domNodes];
      }
    }) as NestArray<DomNode>;
  });

  return domNodes;
};

export const html = (
  url: string,
  filePaths: string[],
  domSelectorNames: string[]
) => {
  try {
    const $ = fetchSync(url);
    const body = new DomNode($("body"), $);

    $("head").append(
      `<script>      
      function genMark() {
        $(".gigantes-mark").remove();
        $(".gigantes-focus").each((i, e) => {
          const height = $(e).css("height");
          const width = $(e).css("width");
          const offset = $(e).offset();
  
          const name = $(e)
            .attr("class")
            .split(" ")
            .find((c) => c.startsWith("name-"))
            .match(/name-(.*)/)[1];

          const $div = $("<div></div>");
          $div.text(name);
          $div.addClass("gigantes-mark");
          $div.css("height", height);
          $div.css("width", width);
          $div.css("position", "absolute");
          $div.css("background-color", "#ff0099aa");
          $div.css("border", "solid 4px #ff0099");
          $div.css("font-weight", "bold");
          $div.css("z-index", "10000");
          $div.css("color", "white");
          $div.css("padding", "4px");
          $div.offset(offset);
  
          $("body").append($div);
        });
      }

      var $element = $("body");
      var lastHeight = $("body").css('height');
      function checkForChanges()
      {
          if ($element.css('height') != lastHeight)
          {
              genMark()
              lastHeight = $element.css('height'); 
          }
      
          setTimeout(checkForChanges, 500);
      }

      $(function(){
        checkForChanges();
      });
      </script>`
    );

    const params: { filePath: string; name: string }[] = [];

    filePaths.forEach((filePath, i) => {
      params.push({
        filePath,
        name: domSelectorNames[i],
      });
    });

    const domSelectors = params.map(({ filePath, name }) => {
      return loadDomSelector(filePath).find(
        (domSelector) => domSelector.name === name
      ).domSelector;
    });

    const name = domSelectorNames.reverse()[0];

    const domNodes = getDomNode(domSelectors, body);
    // console.log(
    //   JSON.stringify(
    //     nestMap(domNodes, (d: DomNode) => ({
    //       tagName: d.tagName,
    //       classNames: d.classNames,
    //     })),
    //     null,
    //     2
    //   )
    // );
    nestMap(domNodes, (domNode: DomNode) => {
      domNode.cheerio.addClass("gigantes-focus");
      domNode.cheerio.addClass(`name-${name}`);
      //   domNode.cheerio.append(`<span class="domselector-name">${name}</span>`);
    });

    return $.html();
  } catch (e) {
    console.error(e);
    return "";
  }
};
